'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var catalogModel = require('@backstage/catalog-model');
var pluginCatalogBackend = require('@backstage/plugin-catalog-backend');
var graphqlRequest = require('graphql-request');

const sourcegraphFileMatchQuery = graphqlRequest.gql`
  query ($search: String!) {
    search(query: $search) {
      results {
        matchCount
        results {
          __typename
          ... on FileMatch {
            repository {
              name
            }
            file {
              url
              content
            }
          }
        }
      }
    }
  }
`;
const parseSourcegraphSearch = (data, providerName) => {
  const parseResults = [];
  data.search.results.results.forEach((result) => {
    const location = {
      type: "url",
      target: `${result.repository.name}/catalog-info.yaml`
    };
    const catalogInfoYamlContent = Buffer.from(result.file.content, "utf8");
    for (const parseResult of pluginCatalogBackend.parseEntityYaml(catalogInfoYamlContent, location)) {
      const parsed = parseResult;
      const annotated = {
        entity: {
          ...parsed.entity,
          metadata: {
            ...parsed.entity.metadata,
            annotations: {
              ...parsed.entity.metadata.annotations,
              [catalogModel.ANNOTATION_LOCATION]: `url:${parsed.location.target}`,
              [catalogModel.ANNOTATION_ORIGIN_LOCATION]: providerName
            }
          }
        },
        locationKey: parsed.location.target
      };
      parseResults.push(annotated);
    }
  });
  return parseResults;
};
class SourcegraphEntityProvider {
  static create(config) {
    return new SourcegraphEntityProvider(config);
  }
  constructor(config) {
    this.config = config;
  }
  getProviderName() {
    return `sourcegraph-provider:${this.config.getString("sourcegraph.orgId")}`;
  }
  async connect(connection) {
    this.connection = connection;
    const endpoint = `https://${this.config.getString("sourcegraph.orgId")}.sourcegraph.com/.api/graphql`;
    this.graphQLClient = new graphqlRequest.GraphQLClient(endpoint, {
      headers: {
        authorization: `token ${this.config.getString("sourcegraph.token")}`
      }
    });
    await this.full();
  }
  async full() {
    if (!this.connection)
      throw new Error("Not initialized");
    if (!this.graphQLClient)
      throw new Error("GraphQL client not initialized");
    const data = await this.graphQLClient.request(sourcegraphFileMatchQuery, {
      search: "file:^catalog-info.yaml$"
    });
    const parsed = parseSourcegraphSearch(data, this.getProviderName());
    await this.connection.applyMutation({
      type: "full",
      entities: parsed.map((entity) => ({
        entity: entity.entity,
        locationKey: entity.locationKey
      }))
    });
  }
  async delta(payload) {
    if (!this.connection)
      throw new Error("Not initialized");
    let toAdd = [];
    let toRemove = [];
    payload.results.forEach(async (result) => {
      if (!this.graphQLClient)
        throw new Error("GraphQL client not initialized");
      const data = await this.graphQLClient.request(sourcegraphFileMatchQuery, {
        search: `file:^catalog-info.yaml$ repo:${result.repository}$`
      });
      const parsed = parseSourcegraphSearch(data, this.getProviderName());
      if (parsed.length) {
        toAdd.push(parsed[0]);
      } else {
        toRemove.push({
          "entity": {
            "apiVersion": "backstage.io/v1alpha1",
            "kind": "Component",
            "metadata": {
              "name": /[^\/]*$/.exec(result.repository)[0]
            }
          },
          "locationKey": `${result.repository}/catalog-info.yaml`
        });
      }
    });
    await this.connection.applyMutation({
      type: "delta",
      added: [...toAdd],
      removed: [...toRemove]
    });
  }
}

exports.SourcegraphEntityProvider = SourcegraphEntityProvider;
//# sourceMappingURL=index.cjs.js.map
